/**
 * @file Signup widgets Controller
 * @author Hussein Arian <hossein@puzzley.ir>
 * @version 1.7.2
 */
var $signupVersion = 1.72;

(function () {
    'use strict';

    angular.module('ngApp').requires.push('ngFileUpload', 'ionic-datepicker', 'ionic');

    angular
        .module('ngApp')
        .controller('profileCtrl', profileController)
        .controller('signupCtrl', SignupController)
        .controller('LoginCtrl', LoginController)
        .controller('EditCtrl', editProfileController)
        .controller('ticketsCtrl', ticketsContoroller)
        .controller('ticketCtrl', ticketContoroller)
        //This is a suitable syntax for minification
        .controller('PlanCtrl', ['$scope', '$http', '$window', 'Auth', '$ionicLoading', '$q', '$ionicPopup', planController])
        .factory('Auth', AuthenticationService)
        .factory('UserService', UserService)
        .filter('join', joinFilter)
        .filter('fixConditionalValue', fixConditionalValueFilter);
    
    function planController($scope, $http, $window, Auth, $ionicLoading, $q, $ionicPopup) {
    
        var fetchPlans = function () {
            return $http({
                method: 'GET',
                url: url + '/api/v1/widget/get/signup/getPlans',
                params: {
                    sl: signup_sl
                }
            }).success(function (result) {
                return result;
            }).error(function (e) {
                return e;
            });
        };

        var loadPlans = function () {
            return fetchPlans().then(function (result) {
                if (typeof result.status === undefined || result.status !== 200) {
                    return;
                }
                if (result.data.success !== true) {
                    return;
                }
                $scope.plans = JSON.parse(result.data.payload.plans);
            });
        }

        var planPaymentRequest = function () {
            if (signupPaymentProvider.providerName === 'cafebazaar' ||
                signupPaymentProvider.providerName === 'myket'
            ) {
                var productId = $scope.selectedPlan.productId;
                var type = $scope.selectedPlan.periodInDays == -1 ? 'buy' : 'subscribe';
                return $q(function (resolve, reject) {
                    return resolve(inappPlanPaymentRequest(productId, type));
                });
            }
            return $http({
                method: 'POST',
                url: url + '/api/v1/widget/post/signup/planPaymentRequest',
                data: {
                    plan: $scope.selectedPlan,
                    user: Auth.User,
                    provider: signupPaymentProvider.providerName,
                },
                params: {sl: signup_sl,},
                responseType: 'json',
            });
        };

        var inappPlanPaymentRequest = function (productId, type) {
            var finalResponse = {
                data: {
                    success: false,
                    payload: {
                        provider: signupPaymentProvider,
                    },
                },
            };
            if(!isDevice) {
                showWarning(lang('signup.not_available_in_similator'));
                finalResponse.data.error = 'This feature is only available in application';
                return finalResponse;
            }
            if(!isOnline()) {
                checkYourInternet();
                finalResponse.data.error = 'Check your internet connection';
                return finalResponse;
            }
            $ionicLoading.show();
            inappbilling.init(function() {
                inappbilling[type](
                    function(result) {
                        verifyInappPlanPayment(result, type, signup_sl);
                    }, function() {
                        showError(lang('signup.payment_failed'));
                        return;
                    },
                    productId
                );
            }, function() {
                showWarning(
                    lang('signup.fail_iap_message1') +
                    signupPaymentProvider.providerName +
                    lang('signup.fail_iap_message2') 
                );
                setTimeout(function () {
                    signupPaymentProvider.providerName === 'cafebazaar' ?
                        window.open('bazaar://login', '_system') :
                        window.open('myket://details', '_system');
                }, 3000);
            },
            { showLog: false },
            signupPaymentProvider.providerName === 'cafebazaar' ?
                'com.farsitel.bazaar' : 'ir.mservices.market',
            signupPaymentProvider.providerName === 'cafebazaar' ?
                'ir.cafebazaar.pardakht.InAppBillingService.BIND' :
                'ir.mservices.market.InAppBillingService.BIND',
            signupPaymentProvider.RSAKey);
        }

        var verifyInappPlanPayment = function(result, type, sl) {
            $ionicLoading.show();
            $http({
                method: 'POST',
                url: url + '/api/v1/widget/post/signup/cafebazaarVerifyPlanPayment?sl=' + sl,
                data: {
                    packageName: result.packageName,
                    productId: result.productId,
                    purchaseToken: result.purchaseToken,
                    provider: signupPaymentProvider.providerName,
                    code: result.code,
                    type: type,
                },
                responseType: 'json'
            })
            .success(function(data) {
                $ionicLoading.hide();
                data.success === true ?
                    planInappPaymentSucceeded():
                    showError(data.error);
            }).error(function() {
                $ionicLoading.hide();
                $ionicPopup.alert({
                    cssClass: 'iap_pupop',
                    title: '',
                    template: lang('signup.fail_iap_confirm'),
                    buttons: [{
                        text: '<b>تلاش مجدد</b>',
                        type: 'button-positive',
                        onTap: function() {
                            verifyInappPlanPayment(result, type, sl);
                        }
                    }, {
                        text: 'بستن',
                        type: 'button-stable'
                    }]
                });
            });
        };
    
        var planInappPaymentSucceeded = function () {
            Auth.UpdateUser();
            $ionicPopup.show({
                cssClass: 'iap_pupop',
                title: lang('signup.payment_successed'),
                buttons: [{
                    text: lang('global.close'),
                    type: 'button-clear button-balanced'
                }],
            });
            window.location.hash = homeHash;
        }

        $scope.init = function () {
            $scope.plans = [];
            loadPlans().then(function () {
                if ($scope.plans.length > 0) {
                    $scope.selectPlan($scope.plans[0]);
                }
            });
        };

        $scope.selectPlan = function (plan) {
            $scope.selectedPlan = plan;
        };

        $scope.purchasePlan = function () {
            planPaymentRequest()
                .then(function (response) {
                    if (typeof response === 'undefined') {
                        return;
                    }
                    if (typeof response.data === 'undefined') {
                        return;
                    }
                    if (!response.data.success) {
                        showWarning(response.data.error);
                        return;
                    }
                    // set cookie for back button in web applications
                    if (!isDevice) {
                        var d = new Date();
                        d.setTime(d.getTime() + (3600000));
                        var expires = "expires=" + d.toUTCString();
                        document.cookie = "backUrl=" + window.location + ";" + expires + ";path=/";
                    }

                    if (!ionic.Platform.isIOS()) {
                        $window.open(response.data.payload.paymentUrl, '_system');
                        ionic.Platform.exitApp();
                        $window.close();
                    } else {
                        var ref = window.open(encodeURI(response.data.payload.paymentUrl), '_blank', "location=yes&clearcache=yes&zoom=no&toolbar=no,status=no,titlebar=no");
                        ref.addEventListener('loadstop', function (event) {
                            if (event.url.search('planPaymentCallback') != -1) {
                                // @todo add other payment
                                if (event.url.search('Status=NOK') != -1) {
                                    // PAYMENT failed
                                    ref.close();
                                } else if (event.url.search('Status=OK') != -1) {
                                    // PAYMENT SUCCESS
                                    Auth.UpdateUser();
                                    planInappPaymentSucceeded();
                                }
                            }
                        });
                    }
                }, function (e) {
                    showWarning(e);
                });
        };
    }

    angular.module('ngApp').service('termsModalService', function($ionicModal) {

        this.showTermsModal = function() {
            $ionicModal.fromTemplateUrl('terms.html', {
                scope: null,
                animation: 'slide-in-up',
                hardwareBackButtonClose: true
            }).then(function(modal) {
                modal.show();
            });
        };
    });

    /**
     * Profile page Controller
     * show user data
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    profileController.$inject = ['$scope', '$http', 'Auth'];
    function profileController($scope, $http, Auth) {

        var fetchPlans = function () {
            return $http({
                method: 'GET',
                url: url + '/api/v1/widget/get/signup/getPlans',
                params: {
                    sl: signup_sl
                }
            }).success(function (result) {
                return result;
            }).error(function (e) {
                return e;
            });
        };

        var loadPlans = function () {
            fetchPlans().then(function (result) {
                if (typeof result.status === undefined || result.status !== 200) {
                    return;
                }
                if (result.data.success !== true) {
                    return;
                }
                $scope.plans = JSON.parse(result.data.payload.plans);
            });
        }

        $scope.isUserExpired = function (user) {
            return (new Date(user.expire_date)) < (new Date());
        }

        $scope.version = $signupVersion;

        $scope.init = function () {
            $scope.plans = [];
            loadPlans();
        };

        $scope.logout = function() {
            Auth.Logout();
        }

        $scope.change_password = function(isValid) {
            if (isValid) {
                Auth.ChangePassword($scope.oldPassword, $scope.newPassword, function(response) {
                    if(response.success) {
                        showSuccess(lang('signup.password_changed_successfully_message'), 2400);
                        setTimeout(function () {
                            window.location.hash = homeHash;
                        }, 2500);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        }
    }

    /**
     * Edit Profile page Controller
     * show input of user data for update
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    editProfileController.$inject = ['$scope', 'Auth', 'ionicDatePicker', 'ionicTimePicker'];
    function editProfileController($scope, Auth, ionicDatePicker, ionicTimePicker) {

        $scope.edit = angular.copy(Auth.User);
        $scope.editimagesets = $scope.edit.profile_pic != '' ? true : false;
        $scope.model.profile_pic = $scope.edit.profile_pic;

        $scope.submitform = function (isValid) {
            if (isValid) {
                var form_data = $('#editForm').serializeObject();
                var password = form_data.password;
                form_data.profile_pic = $scope.model.profile_pic;
                Auth.Update(form_data, function (response) {
                    if(response.success) {
                        showSuccess(lang('signup.successfully_edited'), 1000);
                        setTimeout(function () {
                            var isUserActive = Boolean(response.payload.userIsActive);
                            if (!isUserActive) {
                                Auth.Logout();
                                if(response.payload.verification == 'by_email') {
                                    $ionicPopup.alert({
                                        title: lang('signup.confirm_email'),
                                        template: lang('signup.email_send_successfully'),
                                        okText: lang('signup.confirm_message'),
                                    }).then(function() {
                                        window.location.hash = signupHash;
                                    })
                                } else if(response.payload.verification == 'by_cellphone') {
                                    Auth.username = response.payload.user.username;
                                    Auth.password = password;
                                    window.location.hash = '#/nav/widget/signup/confirmCellphone/'+ signup_sl + '/' + response.payload.user.user_id;
                                }
                            }
                        }, 1100);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        }

        var datevalue = false, timevalue = false;
        $scope.openDatePicker = function(model, time) {
            ionicDatePicker.openDatePicker({
                callback: function (unix, dateString) {
                    datevalue = unix;
                    $scope['edit']['extra_data'][model]['value'] = dateString;
                    if(time) {
                        ionicTimePicker.openTimePicker({
                            callback: function (val) {
                            timevalue = val;
                            var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                            $scope['edit']['extra_data'][model]['value'] = dateString + ' - ' + selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                            },
                            inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
                        });
                    }
                },
                inputDate: datevalue != false ? new Date(datevalue) : new Date(),
            });
        };

        $scope.openTimePicker = function(model){
            ionicTimePicker.openTimePicker({
                callback: function (val) {
                    timevalue = val;
                    var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                    $scope['edit']['extra_data'][model]['value'] = selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                },
                inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
            });
        };

        $scope.showChangePassword = true;
    }

    /**
     * signup form Controller
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    SignupController.$inject = ['$scope', '$rootScope', '$timeout', 'Auth', 'UserService', '$ionicPopup', '$interval', 'ionicDatePicker', 'termsModalService'
    ];
    function SignupController($scope, $rootScope, $timeout, Auth, UserService, $ionicPopup, $interval, ionicDatePicker, termsModalService) {
        $scope.imagesets = false;
        $scope.model = {};
        $scope.confirmMetode = '';
        $scope.time = 90;
        $scope.termsModalService = termsModalService;
        if (!Auth.User.isLogin)
            $rootScope.hideTabs = true;

        $scope.submit = function (isValid) {
            if (isValid) {
                var form_data = $('#signupForm').serializeObject();
                form_data.profile_pic = $scope.model.profile_pic;

                UserService.Register(form_data)
                    .success(function(result) {
                        if (result.success) {
                            showSuccess(lang('signup.successfully_signed_up'), 1000);
                            setTimeout(function () {
                                if (result.payload.isactive) {
                                    Auth.Login(result.payload.username, $scope.model.password, function(response) {
                                        if (response.success) {
                                            $rootScope.hideTabs = false;
                                            window.location.hash = homeHash;
                                        }
                                    });
                                } else {
                                    if(result.payload.verification == 'by_email') {
                                        $ionicPopup.alert({
                                            title: lang('signup.confirm_email'),
                                            template: lang('signup.email_send_successfully'),
                                            okText: lang('signup.confirm_message'),
                                        }).then(function() {
                                            window.location.hash = signupHash;
                                        })
                                    } else if(result.payload.verification == 'by_cellphone') {
                                        Auth.username = result.payload.username;
                                        Auth.password = $scope.model.password;
                                        window.location.hash = '#/nav/widget/signup/confirmCellphone/'+ signup_sl + '/' + result.payload.user_id;
                                    }
                                }
                            }, 1100);
                        } else {
                            showWarning(result.error);
                        }
                    })
                    .error(function (result, statusCode) {
                        if (statusCode == -1) {
                            showWarning(lang('signup.failure_to_communicate_please_try_again'));
                        } else {
                            checkYourInternet();
                        }
                    });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        };

        $scope.validateFile = function(files) {
            if (files != null) {
                var file = files[0];
                if (window.FileReader != null && typeof file != "undefined" && typeof file.type != "undefined") {
                    if (file.size < 5 * 1024 * 1024) {
                        $timeout(function() {
                            var fileReader = new FileReader();
                            fileReader.readAsDataURL(file); // convert the image to data url.
                            fileReader.onload = function(e) {
                                $timeout(function() {
                                    if (file.type.indexOf('image') > -1) {
                                        $scope.imagesets = true;
                                        $scope.model.profile_pic = e.target.result;
                                    } else {
                                        showWarning(lang('signup.the_file_type_is_not_allowed'));
                                    }
                                });
                            }
                        });
                    } else {
                        showWarning(lang('signup.maximum_authorized_volume_is_5m'));
                    }
                } else {
                    $scope.imagesets = false;
                    showWarning(lang('signup.the_file_is_not_valid'));
                }
            }
        };

        $scope.submitConfirm = function (isValid) {
            if (isValid && $scope.code != '') {
                Auth.Confirm($scope.code, $scope.userId, function (response) {
                    if (response.success) {
                        showSuccess(lang('signup.data_registration_and_activation_were_successful'),2000);
                        $timeout(function () {
                            Auth.Login(Auth.username, Auth.password, function (response) {
                                if (response.success) {
                                    $rootScope.hideTabs = false;
                                    window.location.hash = homeHash;
                                } else {
                                    showWarning(lang('signup.login_error_message'));
                                    window.location.hash = signupHash;
                                }
                            });
                        }, 2200);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        }

        $scope.reSendCode = function() {
            if($scope.time <= 5) {
                Auth.ReSendConfirm($scope.userId, function(response) {
                    if (response.success) {
                        showSuccess(lang('signup.the_code_was_sent_again'));
                        $scope.startTimer();
                    } else {
                        showWarning(response.message);
                    }
                })
            }
        }

        $scope.startTimer = function() {
            $scope.time = 90;
            $interval(function() {
                if($scope.time > 0) {
                    $scope.time--;
                } else {
                    $interval.cancel(stop);
                }
            }, 1000);
        }

        var datevalue = false, timevalue = false;
        $scope.openDatePicker = function(model, time) {
            ionicDatePicker.openDatePicker({
                callback: function (unix, dateString) {
                    datevalue = unix;
                    $scope['model'][model] = dateString;
                    if(time) {
                        ionicTimePicker.openTimePicker({
                            callback: function (val) {
                            timevalue = val;
                            var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                            $scope['model'][model] = dateString + ' - ' + selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                            },
                            inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
                        });
                    }
                },
                inputDate: datevalue != false ? new Date(datevalue) : new Date(),
            });
        };

        $scope.openTimePicker = function(model){
            ionicTimePicker.openTimePicker({
                callback: function (val) {
                    timevalue = val;
                    var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                    $scope['model'][model] = selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                },
                inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
            });
        };
    }

    /**
     * Login form Controller
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    LoginController.$inject = ['$scope', '$rootScope', 'Auth', '$ionicPopup', '$ionicScrollDelegate', '$timeout'];
    function LoginController($scope, $rootScope, Auth, $ionicPopup, $ionicScrollDelegate, $timeout) {
        $timeout(function () {
            $ionicScrollDelegate.resize();
        }, 100);
        $rootScope.hideTabs = true;
        $scope.submit = function(isValid) {
            if(isValid) {
                Auth.Login($scope.user.username ,$scope.user.password, function (response) {
                    if (response.success) {
                        $rootScope.hideTabs = false;
                        window.location.hash = homeHash;
                    } else {
                        showWarning(response.message, 2000);
                        setTimeout(function () {
                            if (typeof response.payload.user_not_active != 'undefined') {
                                if (response.payload.verification == 'by_email') {
                                    $ionicPopup.alert({
                                        title: lang('signup.confirm_email'),
                                        template: lang('signup.email_send_successfully'),
                                        okText: lang('signup.confirm_message'),
                                    }).then(function () {
                                        window.location.hash = signupHash;
                                    })
                                } else if (response.payload.verification == 'by_cellphone') {
                                    Auth.username = $scope.user.username;
                                    Auth.password = $scope.user.password;
                                    window.location.hash = '#/nav/widget/signup/confirmCellphone/' + signup_sl + '/' + response.payload.user_id;
                                }
                            }
                        }, 2100);
                    }
                });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        };

        $scope.forget_password = function(isValid) {
            if(isValid) {
                Auth.ForgetPassword($scope.user_identity , function (response) {
                    if(response.success){
                        $ionicPopup.alert({
                            title: '',
                            template: response.message,
                            buttons: [{
                                text: lang('signup.confirm_message'),
                                type: 'button-default'
                            }]
                        });
                        window.location.hash = signupHash;
                    }
                });
            } else {
                showWarning(lang('signup.form_invalid'));
            }
        }
    }

    /**
      * Tickets Controller
      * ticket list view controller
      */
    ticketsContoroller.$inject = ['$scope', '$http'];
    function ticketsContoroller($scope, $http) {
		var vm = this;
		vm.loadMore = loadMore;
		vm.refresh = refresh;
		vm.noMoreItemsAvailable = true;
		vm.infiniteScrollLoading = false;
		vm.message = '';
		var length = 20;

		$scope.$watch('sl', function () {
			getTickets();
		});
		
		function getTickets() {
			$http.post(url + '/api/v1/widget/post/signup/getTickets', {
				sl: $scope.sl,
				length: length
			})
				.success(function (result) {
					if (result.success) {
                        vm.tickets = result.payload.tickets;
                        vm.message = result.message;
						if (result.payload.tickets.length == length) {
							vm.noMoreItemsAvailable = false;
						}
					}
					$scope.$broadcast('scroll.refreshComplete');
				})
				.error(function () {
					checkYourInternet();
					$scope.$broadcast('scroll.refreshComplete');
				});
		}

		function refresh() {
			getTickets();
		}
		
		function loadMore() {
			if (!vm.noMoreItemsAvailable) {
				vm.noMoreItemsAvailable = true;
				var start = vm.tickets.length;
				if (isOnline()) {
					vm.infiniteScrollLoading = true;
					var data = {
						sl: $scope.sl,
						start: start,
						length: length
					};
					$http({
							method: 'POST',
							noLoading: true,
							url: url + '/api/v1/widget/post/signup/getTickets',
							data: data
						})
						.success(function (result) {
							if (result.success) {
								for (var i = 0; i < result.payload.tickets.length; i++) {
									vm.tickets.push(result.payload.tickets[i]);
								}
								vm.noMoreItemsAvailable = result.payload.tickets.length < length;
							}
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						})
						.error(function () {
							console.error('ERR', err.status);
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						});
				}
			} else {
				return;
			}
		}
    }
    

    /**
     * Ticket Controller
     * single ticket view Controller
     */
    ticketContoroller.$inject = ['$scope', '$rootScope', '$http'];
    function ticketContoroller($scope, $rootScope, $http) {
		var vm = this;
		vm.ticket = {};
		vm.sendTicket = sendTicket;
		vm.usertickets = [];

        // on init
        $scope.$watch('ticket_id', function () {
            if ($scope.ticket_id) {
                $rootScope['ticket' + $scope.ticket_id] = 'visited';
                $rootScope.user.unread_tickets_count -= $scope.unread_ticket_count;
                $rootScope.user.unread_tickets_count = Math.max($rootScope.user.unread_tickets_count == undefined ? 0 : $rootScope.user.unread_tickets_count, 0);
            }
		});

		function sendTicket(valid) {
		    if (valid) {
		        var fd = new FormData();
		        for (var key in vm.ticket) {
		            fd.append(key, vm.ticket[key]);
		        }
		        fd.append('sl', $scope.sl);

		        $http.post(url + '/api/v1/widget/post/signup/postTicket', fd, {
		                withCreadential: true,
		                headers: {
		                    'Content-Type': undefined,
		                },
		                transformRequest: angular.identity
		            })
		            .success(function (result) {
		                if (result.success) {
		                    vm.ticket = {
		                        title: '',
		                        content: '',
		                        attach: ''
                            };
                            vm.usertickets.push(result.payload.ticket);
                            if (typeof result.payload.sl != "undefined")
                                $scope.sl = result.payload.sl;
                            showToast(lang('signup.your_message_has_been_successfully_recorded'));
                            $scope.fm.$setPristine();
                            $scope.fm.$setUntouched();
		                } else {
		                    showToast(result.message);
		                }
		            })
		            .error(function () {
		                showToast(lang('signup.failure_to_communicate_please_try_again'));
		            });
		    } else {
		        showToast(lang('signup.form_invalid'));
		        return;
		    }
		}
	}
    
    /**
     * service for make http call for login and Logut
     * @type {factory}
     * @param {UserService} UserService : keeps user data
     * @return {object} AuthenticationService
     */
    AuthenticationService.$inject = ['$rootScope', '$http', '$timeout', 'localStorage', 'UserService'];
    function AuthenticationService($rootScope, $http, $timeout, localStorage, UserService) {
        var Auth = {};
        Auth.username = '';
        Auth.password = '';
        Auth.User = {isLogin : false, accessible_pages : default_access_page};
        Auth.Login = Login;
        Auth.Check = Check;
        Auth.Logout = Logout;
        Auth.CanAccess = CanAccess;
        Auth.SetCredentials = SetCredentials;
        Auth.Update = Update;
        Auth.Confirm = Confirm;
        Auth.ReSendConfirm = ReSendConfirm;
        Auth.ForgetPassword = ForgetPassword;
        Auth.ChangePassword = ChangePassword;
        Auth.UpdateUser = UpdateUser;
        Auth.init = init;

        (function(){
            Auth.init();
        })();

        return Auth;

        function init() {
            Auth.User = localStorage.all('user');
            $rootScope.user = Auth.User;
            if(Auth.User.isLogin){
                $http.defaults.headers.common['X-auth-token'] = Auth.User.token;
                $timeout(function () {
                    UpdateUser();
                }, 4000);
            } else {
                Auth.User.isLogin = false;
                Auth.User.accessible_pages = default_access_page;
                $timeout(function () {
                    UserService.UpdateDefaultAccessPage().success(function (result) {
                        if (result.success) {
                            Auth.User.accessible_pages = result.payload.accessible_pages;
                        }
                    });
                }, 2000);
            }
        }

        function UpdateUser() {
            UserService.UpdateUserInfo().success(function (result) {
                if (result.success) {
                    Auth.SetCredentials(result.payload.user, result.payload.token);
                } else {
                    if (typeof result.payload.shouldLogout !== 'undefined' &&
                        result.payload.shouldLogout) {
                        Auth.Logout();
                    }
                }
            });
        }

        function Login(username, password, callback) {
            var response;
            UserService.LoginWithUsername(username, password)
                .success(function (result) {
                    if (result.success) {
                        Auth.SetCredentials(result.payload.user, result.payload.token);
                        response = { success: true };
                    } else {
                        response = {
                            success: false,
                            message: result.error,
                            payload: result.payload
                        };
                    }
                    callback(response);
                })
                .error(function (result){
                    callback({ success: false, message: lang('signup.failure_to_communicate_please_try_again') });
                    console.error(result);
                });
        }

        function Check() {
            return Auth.User.isLogin;
        }
        function CanAccess(pageid) {
            if (Auth.User.accessible_pages == null) {
                return true;
            }
            return (
                typeof pageid !== 'undefined') &&
                (Auth.User.accessible_pages.includes(Number(pageid)) || (Auth.User.accessible_pages.includes(String(pageid)))
            );
        }

        function Logout() {
            UserService.Logout();
            Auth.User.isLogin = false;
            Auth.User.token = '';
            Auth.User.accessible_pages = default_access_page;
            localStorage.save(Auth.User,'user');
        }

        function SetCredentials(user, token) {
            Auth.User = user;
            Auth.User.isLogin = true;
            Auth.User.token = token;
            $rootScope.user = Auth.User;
            $http.defaults.headers.common['X-auth-token'] = token;
            localStorage.save(Auth.User,'user');
        }

        function Update(user, callback) {
            var response;
            UserService.Update(user)
                .success(function (result) {
                    result.payload = typeof result.payload === 'undefined' ? [] : result.payload;
                    if (result.success) {
                        Auth.SetCredentials(result.payload.user, result.payload.token);
                        response = { success: true, payload: result.payload };
                    } else {
                        response = { success: false, message: result.error, payload: result.payload };
                    }
                    callback(response);
                })
                .error(function (result, statusCode) {
                    result.payload = typeof result.payload === 'undefined' ? [] : result.payload;
                    var message = statusCode == -1 ? lang('signup.error_in_sending_information') : lang('signup.error_in_updating');
                    callback({ success: false, message: message, payload: result.payload });
                });
        }

        function Confirm(code, user_id, callback) {
            var response;
            UserService.ConfirmCellphone(code, user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true };
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function (){
                    callback({ success: false, message: lang('signup.error_in_sending_information') });
                });
        }

        function ReSendConfirm(user_id, callback) {
            var response;
            UserService.resendconfirmcode(user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true };
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function () {
                    callback({ success: false, message: lang('signup.error_in_sending_information') });
                });
        }

        function ForgetPassword(user_id, callback) {
            var response;
            UserService.ForgetPassword(user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true, message: result.payload.message };
                    }
                    callback(response);
                })
                .error(function (){
                    checkYourInternet();
                    callback({ success: false, message: lang('signup.error_in_sending_information') });
                });
        }

        function ChangePassword(oldPass, newPass, callback) {
            var response;
            UserService.ChangePassword(oldPass, newPass)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true};
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function (result){
                    callback({ success: false, message: lang('signup.error_in_sending_information') });
                });
        }
    }

    /**
     * service for make API call to users api
     * @type {factory}
     * @param {$http}
     * @return {object} UserService
     */
    UserService.$inject = ['$http'];
    function UserService($http) {
        var service = {};

        service.LoginWithUsername = LoginWithUsername;
        service.Register = Register;
        service.Update = Update;
        service.ConfirmCellphone = ConfirmCellphone;
        service.resendconfirmcode = resendconfirmcode;
        service.UpdateUserInfo = UpdateUserInfo;
        service.Logout = Logout;
        service.ForgetPassword = ForgetPassword;
        service.ChangePassword = ChangePassword;
        service.UpdateDefaultAccessPage = UpdateDefaultAccessPage;

        return service;

        function LoginWithUsername(username, password) {
            return $http.post(url + '/api/v1/widget/post/signup/Login', {
                        username : username,
                        password : password,
                        phone_id : phone_id,
                        sl : signup_sl
                    });
        }

        function Register(user) {
            return $http.post(url + '/api/v1/widget/post/signup/Register', user);
        }

        function Update(user) {
            return $http.post(url + '/api/v1/widget/post/signup/editUser', user);
        }

        function ConfirmCellphone(code, user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/confirmByCellphone', {
                        code : code,
                        user_id : user_id,
                        sl : signup_sl
                    });
        }

        function resendconfirmcode(user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/resendConfirmCodeToCellphone', {
                        user_id : user_id,
                        sl : signup_sl
                    });
        }

        function UpdateUserInfo() {
            return $http.post(url + '/api/v1/widget/post/signup/updateUserInfo', {
                        sl : signup_sl
                    },{
                        noLoading : true
                    });
        }

        function Logout() {
            return $http.post(url + '/api/v1/widget/post/signup/logout', {
                        sl : signup_sl
                    },{
                        noLoading : true
                    });
        }

        function ForgetPassword(user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/resetPassword', {
                        sl : signup_sl,
                        user_identity : user_id
                    });
        }

        function ChangePassword(oldPass, newPass) {
            return $http.post(url + '/api/v1/widget/post/signup/changePassword', {
                        sl : signup_sl,
                        old_password : oldPass,
                        new_password : newPass
                    });
        }

        function UpdateDefaultAccessPage() {
            return $http.post(url + '/api/v1/widget/post/signup/updateDefaultAccessPage', {
                sl: signup_sl
            }, {
                noLoading: true
            });
        }

    }
    /**
     * filter to convert conditional values to miningfull string
     * @type {filter}
     * @param {string} string string to convert
     * @return {function} filter
     */
    function fixConditionalValueFilter() {
        return function fixConditionalValue(string) {
            if (Array.isArray(string) || typeof string == 'undefined') {
                return string;
            }
            var originalString = string.toString();
            try {
                string = string.toLowerCase();
            } catch (e) {
                string = '';
            }
            return (
                    string == 'on'    || 
                    string == 'true'  || 
                    string == '1'
                   )    ? lang('signup.yes_message') :
                   (
                    string == 'off'   || 
                    string == 'false' || 
                    string == '0'
                   )   ? lang('signup.no_message') : originalString;
        };
    }
    /**
     * filter to join array data as string with giving prop
     * @type {filter}
     * @param {array} array string or array to convert
     * @param {sprator} string sprator sign
     * @param {prop} string prop of array to show
     * @return {function} filter
     */
    function joinFilter() {
        return function join(array, separator, prop) {
            if (!Array.isArray(array)) {
                return array; // if not array return original
            }

            return (!angular.isUndefined(prop) ? array.map(function (item) {
                return item[prop];
            }) : array).join(separator);
        };
    }
})();

document.addEventListener("deviceready", function(){
	phone_id = device.uuid;
});

window.planPaymentSucceed = function () {
    //
};

window.planPaymentFailed = function () {
    //
};

$.fn.serializeObject = function () {
    var o = {};
    var a = this.serializeArray();
    // console.log(a);
    $.each(a, function () {
        if (o[this.name] !== undefined) {
            if (!o[this.name].push) {
                o[this.name] = [o[this.name]];
            }
            o[this.name].push(this.value || '');
        } else {
            o[this.name] = this.value || '';
        }
    });
    return o;
};